@@ -1,4 +1,4 @@
-VERSION = '0.5'
+VERSION = '0.6'
 
 from .design_flute import Design_tapered_pflute, Design_straight_pflute, Design_tapered_folk_flute, Design_straight_folk_flute
 from .make_flute import Make_flute
@@ -9,7 +9,7 @@ from .make_shawm import Make_shawm
 from .make_mouthpiece import Make_mouthpiece
 from .make_bauble import Make_bauble
 
-from .design_whistle import Design_folk_whistle
+from .design_whistle import Design_folk_whistle, Design_three_hole_whistle
 from .make_whistle import Make_whistle
 
 from .make_panpipe import Make_panpipe
@@ -31,6 +31,7 @@ def main():
             
             'Whistles',
             Design_folk_whistle,
+            Design_three_hole_whistle,
             Make_whistle,
             
             'Shawms',
