@@ -7,6 +7,7 @@ import shape
 
 CODE = r"""
 #include <CGAL/Exact_predicates_exact_constructions_kernel.h>
+#include <CGAL/Simple_homogeneous.h>
 
 #include <CGAL/Nef_polyhedron_3.h>
 #include <CGAL/Polyhedron_incremental_builder_3.h>
@@ -28,7 +29,8 @@ CODE = r"""
 //typedef CGAL::Exact_predicates_exact_constructions_kernel Kernel;
 //typedef CGAL::Extended_homogeneous<CGAL::Gmpz>            Kernel;
 //typedef CGAL::Cartesian<CGAL::Gmpq>                       Kernel;
-typedef CGAL::Homogeneous<CGAL::Gmpz>                     Kernel;
+//typedef CGAL::Homogeneous<CGAL::Gmpz>                     Kernel;
+typedef CGAL::Simple_homogeneous<CGAL::Gmpz>                Kernel;
 
 typedef Kernel::Point_3                                   Point_3;
 typedef Kernel::Vector_3                                  Vector_3;
@@ -93,6 +95,37 @@ M = cpp.Module(os.path.join(os.path.expanduser('~'),'.demakein'), CODE, CMAKE_CO
 def int_round(value):
     return int(math.floor(value+0.5))
 
+def rotation_matrix(x,y,z,angle):
+    length = math.sqrt(x*x+y*y+z*z)
+    x = x/length
+    y = y/length
+    z = z/length
+    angle = angle * (math.pi/180.0)
+    c = math.cos(angle)
+    s = math.sin(angle)
+    return [
+        [ c+x*x*(1-c),   x*y*(1-c)-z*s, x*z*(1-c)+y*s ],
+        [ y*x*(1-c)+z*s, c+y*y*(1-c),   y*z*(1-c)-x*s ],
+        [ z*x*(1-c)-y*s, z*y*(1-c)+x*s, c+z*z*(1-c)   ],
+    ]
+
+def transform_point_3(matrix, point):
+    return tuple(
+        sum( matrix[i][j] * point[j] for j in (0,1,2) )
+        for i in (0,1,2)
+    )
+
+def extent_3(points):
+    xs = []
+    ys = []
+    zs = []
+    for point in points:
+        xs.append(point[0])
+        ys.append(point[1])
+        zs.append(point[2])
+    return shape.Limits(min(xs),max(xs),min(ys),max(ys),min(zs),max(zs))
+
+
 class Shape(object):
     """ Wrapper around Nef_polyhedron """
     
@@ -185,23 +218,30 @@ class Shape(object):
         return Shape(M('CGAL::minkowski_sum_3(a,b)',a=self.nef,b=other.nef))
 
     def rotate(self, x,y,z,angle, accuracy=1<<16):
-        length = math.sqrt(x*x+y*y+z*z)
-        x = x/length
-        y = y/length
-        z = z/length
-        angle = angle * (math.pi/180.0)
-        c = math.cos(angle)
-        s = math.sin(angle)
-        rot = [
-            [ c+x*x*(1-c),   x*y*(1-c)-z*s, x*z*(1-c)+y*s ],
-            [ y*x*(1-c)+z*s, c+y*y*(1-c),   y*z*(1-c)-x*s ],
-            [ z*x*(1-c)-y*s, z*y*(1-c)+x*s, c+z*z*(1-c)   ],
-        ]
-        
+        if not angle: return    
+        rot = rotation_matrix(x,y,z,angle)        
         transform = M.call('Aff_transformation_3',
             *[ int_round(item*accuracy) for row in rot for item in row ] + [ accuracy ]
         )
         M.do('a.transform(b)',a=self.nef, b=transform)
+        
+        # Rotation of a nef appears to be extremely memory hungry
+        # TODO: retain exact representation
+        #
+        #verts = [ ]
+        #vert_index = { }
+        #tris = [ ]
+        #for tri in self.iter_triangles():
+        #    tri_verts = [ ]
+        #    for point in tri:
+        #        if point not in vert_index:
+        #            vert_index[point] = len(verts)
+        #            verts.append(point)
+        #        tri_verts.append(vert_index[point])
+        #    tris.append(tri_verts)
+        #
+        #self.nef = create([ shape.transform_point_3(rot,item) for item in verts ], tris).nef
+        
     
     def move(self, x,y,z, accuracy=1<<16):
         transform = M('Aff_transformation_3(CGAL::TRANSLATION,Vector_3(a,b,c,d))',
@@ -209,6 +249,10 @@ class Shape(object):
         )
         M.do('a.transform(b)',a=self.nef, b=transform)
 
+    def position_nicely(self):
+        e = self.extent()
+        self.move(-0.5*(e.xmin+e.xmax),-0.5*(e.ymin+e.ymax),-e.zmin)
+
     def mask(self, res):
         import mask
         
@@ -270,21 +314,38 @@ def create(verts, tris, name=None, accuracy=1<<16):
     for tri in tris:
         assert len(tri) == 3
     
-    seen = set()
+    #seen = set()
+    vert_hash = { }
+    vert_remap = [ ]
+    n_vert_merges = 0
     for vert in verts:
         [x,y,z] = [ int_round(item*accuracy) for item in vert ]
-        assert (x,y,z) not in seen
-        seen.add((x,y,z))
-        point = M.new('Point_3(a,b,c,d)',a=x,b=y,c=z,d=accuracy)
-        M.do('a.points.push_back(b)',a=builder,b=point)
+        #assert (x,y,z) not in seen
+        #seen.add((x,y,z))
+        if (x,y,z) not in vert_hash:
+            point = M.new('Point_3(a,b,c,d)',a=x,b=y,c=z,d=accuracy)
+            M.do('a.points.push_back(b)',a=builder,b=point)
+            vert_hash[(x,y,z)] = len(vert_hash)
+        else:
+            n_vert_merges += 1
+        vert_remap.append(vert_hash[(x,y,z)])
     
+    n_tri_losses = 0
     for tri in tris:
-        assert tri[0] != tri[1] and tri[0] != tri[2] and tri[1] != tri[2]
+        #assert tri[0] != tri[1] and tri[0] != tri[2] and tri[1] != tri[2]
+        tri = [ vert_remap[i] for i in tri ]
+        if tri[0] == tri[1] or tri[0] == tri[2] or tri[1] == tri[2]:
+            n_tri_losses += 1
+            continue
+        
         vec = M.new('std::vector<int>()')
         for item in tri: 
             M.do('a.push_back(b)',a=vec,b=item)
         M.do('a.facets.push_back(b)',a=builder,b=vec)
     
+    if n_vert_merges or n_tri_losses:
+        print 'Verts merged: %d of %d, triangles discarded: %d of %d' % (n_vert_merges, len(verts), n_tri_losses, len(tris))
+    
     M.do('a.delegate(b)',a=polyhedron,b=builder)
     
     assert M('a.is_closed()',a=polyhedron), 'Polyhedron is not closed'        
