@@ -392,7 +392,7 @@ class Instrument_designer(config.Action_with_output_dir):
     #n-2 initial positions of bore kinks, as fraction of initial length
     @property
     def initial_inner_fractions(self):
-        return [ (i+1.0)/(len(self.inner_diameters)-1) for i in range(len(self.inner_diameters)-2) ]
+        return [ (i+1.0)/(len(self.inner_diameters)-1) for i in xrange(len(self.inner_diameters)-2) ]
 
     @property
     def min_inner_fraction_sep(self):
@@ -400,7 +400,7 @@ class Instrument_designer(config.Action_with_output_dir):
 
     @property
     def initial_outer_fractions(self):
-        return [ (i+1.0)/(len(self.outer_diameters)-1) for i in range(len(self.outer_diameters)-2) ]
+        return [ (i+1.0)/(len(self.outer_diameters)-1) for i in xrange(len(self.outer_diameters)-2) ]
 
     @property
     def min_outer_fraction_sep(self):
@@ -408,7 +408,7 @@ class Instrument_designer(config.Action_with_output_dir):
 
     @property
     def initial_hole_fractions(self):
-        return [ (i+3.0)/(self.n_holes+2) * 0.5 for i in range(self.n_holes) ]
+        return [ (i+3.0)/(self.n_holes+2) * 0.5 for i in xrange(self.n_holes) ]
 
     @property
     def initial_hole_diameter_fractions(self):
@@ -420,6 +420,10 @@ class Instrument_designer(config.Action_with_output_dir):
 
     @property
     def initial_state_vec(self):
+        assert len(self.initial_hole_fractions) == self.n_holes, 'initial_hole_fractions has wrong length'
+        assert len(self.initial_hole_diameter_fractions) == self.n_holes, 'initial_hole_diameter_fractions has wrong length'
+        assert len(self.initial_inner_fractions) == len(self.inner_diameters)-2, 'initial_inner_fractions has wrong length'
+        assert len(self.initial_outer_fractions) == len(self.outer_diameters)-2, 'initial_outer_fractions has wrong length'
         return (
             [ 1.0 ] +
             self.initial_hole_fractions + 
