@@ -5,17 +5,44 @@ from nesoni import config
 
 from . import design, profile
 
-@config.help("""\
-Design a whistle with pennywhistle fingering.
-""")
 @config.Float_flag('tweak_gapextra')
-class Design_folk_whistle(design.Instrument_designer):
+class Design_whistle(design.Instrument_designer):
     def get_whistle_maker(self):
         from . import make_whistle
         return make_whistle.Make_whistle_head(bore=self.inner_diameters[-1])
         
     closed_top = False
+
+    # For gap_width 0.5, gap_length 0.25, from soprano
+    # tweak_gapextra = 0.37
     
+    # For gap_width 0.666, gap_length 0.4, from soprano
+    tweak_gapextra = 0.01
+    
+    def patch_instrument(self, inst):
+        inst = copy.copy(inst)
+        inst.true_length = inst.length
+        inst.true_inner = inst.inner
+        
+        maker = self.get_whistle_maker()
+        diameter = maker.effective_gap_diameter()                
+        length = maker.effective_gap_height()
+        length += diameter * self.tweak_gapextra  # Virtual extra length seems to be needed, as with flute
+                                                  # Not so much if hole is large though
+        
+        inst.inner = profile.Profile(
+            inst.inner.pos + [ inst.length + length ],
+            inst.inner.low + [ diameter ],
+            inst.inner.high[:-1] + [ diameter, diameter ],
+            )
+        inst.length += length
+        return inst
+
+
+@config.help("""\
+Design a whistle with pennywhistle fingering.
+""")
+class Design_folk_whistle(Design_whistle):    
     initial_length = design.wavelength('D4') * 0.5
     
     min_hole_diameters = design.sqrt_scaler([ 3.0 ]*6)
@@ -60,29 +87,45 @@ class Design_folk_whistle(design.Instrument_designer):
         ('D6',   [1,1,1,1,1,1]),
     ]
     
-    # For gap_width 0.5, gap_length 0.25, from soprano
-    # tweak_gapextra = 0.37
+
+@config.help("""\
+Design a three hole pipe, i.e. the pipe in pipe-and-tabor.
+""")
+class Design_three_hole_whistle(Design_whistle):    
+    initial_length = design.wavelength('D3') * 0.5
     
-    # For gap_width 0.666, gap_length 0.4, from soprano
-    tweak_gapextra = 0.01
+    min_hole_diameters = design.sqrt_scaler([ 3.0 ]*3)
+    max_hole_diameters = design.sqrt_scaler([ 20.0 ]*3)
     
-    def patch_instrument(self, inst):
-        inst = copy.copy(inst)
-        inst.true_length = inst.length
-        inst.true_inner = inst.inner
-        
-        maker = self.get_whistle_maker()
-        diameter = maker.effective_gap_diameter()                
-        length = maker.effective_gap_height()
-        length += diameter * self.tweak_gapextra  # Virtual extra length seems to be needed, as with flute
-                                                  # Not so much if hole is large though
-        
-        inst.inner = profile.Profile(
-            inst.inner.pos + [ inst.length + length ],
-            inst.inner.low + [ diameter ],
-            inst.inner.high[:-1] + [ diameter, diameter ],
-            )
-        inst.length += length
-        return inst
+    #min_hole_spacing = [ -20.0, -20.0 ]
+    
+    initial_hole_fractions = [ 0.1,0.15,0.2 ]
+    
+    inner_diameters = design.sqrt_scaler([ 20.0, 20.0 ])
+    
+    outer_add = True
+    outer_diameters = design.sqrt_scaler([ 8.0, 8.0 ])
+    
+    initial_inner_fractions = [ ]
+    
+    initial_outer_fractions = [ ]
+    #min_outer_fraction_sep = [ 0.15, 0.8 ]
+    
+    min_inner_fraction_sep = [ 0.01 ] * 8
+
+    fingerings = [
+        ('D4', [1,1,1]),
+        ('E4', [0,1,1]),
+        ('F#4',[0,0,1]),
+        ('G4', [0,0,0]),
+        ('A4', [1,1,1]),
+        ('B4', [0,1,1]),
+        ('C#5',[0,0,1]),
+        ('D5', [0,0,0]),
+        ('D5', [1,1,1]),
+        ('E5', [0,1,1]),
+        ('F#5',[0,0,1]),
+        ('G5', [0,0,0]),
+    ]
 
 
