@@ -134,29 +134,56 @@ class Pack(object):
                 temp.move(x,y,0)
                 upper.add(temp)
             
-        #bol = int(self.xsize / 20.0)
-        #bol_width = 1.0
-        #    # 0.5 seemed to allow shifting on a sop flute
-        #bol_height = 1.0 
-        #    # 0.5 was too low, middle piece tore loose in final pass of
-        #    # contour finishing (x-scan of horizontal-like surfaces)
-        #xmins = [ x for x,y,packable in self.items ]
-        #xmaxs = [ x+packable.extent.xmax for x,y,packable in self.items ]
-        #margin = bol_width+bit_diameter+0.25
+        bol = int(self.xsize / 10.0)
+        bol_width = 1.0
+            # 0.5 seemed to allow shifting on a sop flute
+        bol_height = 1.0 
+            # 0.5 was too low, middle piece tore loose in final pass of
+            # contour finishing (x-scan of horizontal-like surfaces)
+        xmins = [ x for x,y,packable in self.items ]
+        xmaxs = [ x+packable.extent.xmax for x,y,packable in self.items ]
+        margin = bol_width+bit_diameter+0.25
+        
+        potential_bols = [ ((i+0.5)*xsize/bol, 0,ysize) for i in xrange(bol) ]
+        bols = [ ]
+        while potential_bols:
+            pos, y_low, y_high = potential_bols.pop()
+            for i, (x,y,packable) in enumerate(self.items):
+                if pos-margin <= xmins[i] <= pos+margin or \
+                   pos-margin <= xmaxs[i] <= pos+margin:
+                   low = y-bit_diameter
+                   high = y+packable.extent.ymax+bit_diameter
+                   if low <= y_low and y_high <= high:
+                       break
+                   elif y_low < low and high < y_high:
+                       potential_bols.append((pos, y_low, low))
+                       potential_bols.append((pos, high, y_high))
+                       break
+                   elif y_low < low < y_high <= high:
+                       potential_bols.append((pos, y_low, low))
+                       break
+                   elif low < y_low < high <= y_high:
+                       potential_bols.append((pos, high, y_high))
+                       break
+            else:
+                bols.append((pos,y_low,y_high))
+        
         #for i in range(bol):
         #    pos = (i+0.5)*xsize/bol
-        #    
+            
         #    if any( pos-margin <= item <= pos+bol_width for item in xmins ) or \
-        #       any( pos-bol_width <= item <= pos+margin for item in xmins ):
+        #       any( pos-bol_width <= item <= pos+margin for item in xmaxs ):
         #        continue
-        #    
-        #    upper.add(shape.block(
-        #        pos-bol_width*0.5, pos+bol_width*0.5,
-        #        0,ysize,
-        #        #-bit_diameter*0.6, bol_height,
-        #        0,bol_height,
-        #    ))     
-        #
+        
+        for pos, y_low, y_high in bols:    
+            upper.add(shape.block(
+                pos-bol_width*0.5, pos+bol_width*0.5,
+                y_low, y_high,
+                #0,ysize,
+                #-bit_diameter*0.6, bol_height,
+                0,bol_height,
+            ))     
+        
         
         print 'Underside'
         
